%let Dir D:\Dropbox\Research\Samuelson\Submit\Table1;

PROC IMPORT OUT= WORK.FirmInfo 
            DATAFILE= "&Dir\FirmInfo.txt" 
            DBMS=TAB REPLACE;
     GETNAMES=YES;
     DATAROW=2; 
RUN;

/********************************************************
Check for continuous registration in CRSP
********************************************************/

/********************************************************
Discontinous Firms
********************************************************/

data DCnt_CRSP;
set FirmInfo;
if Nyprc ^= Edyear-Styear+1;
group = 0;
if Nydiv > 0 then group = 1;
run;

proc sort;
by group;
run;

title "Discountinous Firms";
proc means n sum;
var nyprc;
by group;
output out=TB1_DSCF n=nf sum=nfy;
run;

Data TB1_DSCF;
set TB1_DSCF;
Cat = 'Discontinous Whole Period';
run;

Data TB1;
set TB1_DSCF;
run;

/********************************************************
Basic Information for Firm age <=10
********************************************************/

data Cnt_CRSP_1;
set FirmInfo;
if Nyprc = Edyear-Styear+1;
if Nyprc <=10;
group = 0;
if Nydiv > 0 then group = 1;
run;

proc sort;
by group;
run;
title "Basic Information for Firm age <=10";
proc means n sum;
var nyprc;
by group;
output out=TB1_CF1 n=nf sum=nfy;
run;

Data TB1_CF1;
set TB1_CF1;
Cat = 'Continuous_le10';
run;

Data TB1;
set TB1 TB1_CF1;
run;

/********************************************************
Basic Information for 10 < Firm age <= 20
********************************************************/

data Cnt_CRSP_2;
set FirmInfo;
if Nyprc = Edyear-Styear+1;
if Nyprc <=20;
if Nyprc > 10;
group = 0;
if Nydiv > 0 then group = 1;
run;

proc sort;
by group;
run;
title "Basic Information for 10 < Firm age <= 20";

proc means n sum;
var nyprc;
by group;
output out=TB1_CF2 n=nf sum=nfy;
run;

Data TB1_CF2;
set TB1_CF2;
Cat = 'Continuous_le20';
run;

Data TB1;
set TB1 TB1_CF2;
run;

/********************************************************
Basic Information for 20 < Firm age <= 40
********************************************************/

data Cnt_CRSP_3;
set FirmInfo;
if Nyprc = Edyear-Styear+1;
if Nyprc <=40;
if Nyprc > 20;
group = 0;
if Nydiv > 0 then group = 1;
run;

proc sort;
by group;
run;
title "Basic Information for 20 < Firm age <= 40";

proc means n sum;
var Nyprc;
by group;
output out=TB1_CF3 n=nf sum=nfy;
run;

Data TB1_CF3;
set TB1_CF3;
Cat = 'Continuous_le40';
run;

Data TB1;
set TB1 TB1_CF3;
run;

/********************************************************
Basic Information for 40 < Firm age <= 60
********************************************************/

data Cnt_CRSP_4;
set FirmInfo;
if Nyprc = Edyear-Styear+1;
if Nyprc <=60;
if Nyprc > 40;
group = 0;
if Nydiv > 0 then group = 1;
run;

proc sort;
by group;
run;
title "Basic Information for 40 < Firm age <= 60";

proc means n sum;
var nyprc;
by group;
output out=TB1_CF4 n=nf sum=nfy;
run;

Data TB1_CF4;
set TB1_CF4;
Cat = 'Continuous_le60';
run;

Data TB1;
set TB1 TB1_CF4;
run;

/********************************************************
Basic Information for 60 < Firm age <= 80
********************************************************/

data Cnt_CRSP_5;
set FirmInfo;
if Nyprc = Edyear-Styear+1;
if Nyprc <=80;
if Nyprc > 60;
group = 0;
if Nydiv > 0 then group = 1;
run;

proc sort;
by group;
run;
title "Basic Information for 60 < Firm age <= 80";

proc means n sum;
var nyprc;
by group;
output out=TB1_CF5 n=nf sum=nfy;
run;

Data TB1_CF5;
set TB1_CF5;
Cat = 'Continuous_le80';
run;

Data TB1;
set TB1 TB1_CF5;
run;

/********************************************************
Basic Information for 80 < Firm age <= 90
********************************************************/

data Cnt_CRSP_6;
set FirmInfo;
if Nyprc = Edyear-Styear+1;
if Nyprc <= 90;
if Nyprc > 80;
group = 0;
if Nydiv > 0 then group = 1;
run;

proc sort;
by group;
run;
title "Basic Information for 80 < Firm age <= 90";

proc means n sum;
var nyprc;
by group;
output out=TB1_CF6 n=nf sum=nfy;
run;

Data TB1_CF6;
set TB1_CF6;
Cat = 'Continuous_le90';
run;

Data TB1;
set TB1 TB1_CF6;
run;

/********************************************************
Basic Information for Firm age > 90
********************************************************/

data Cnt_CRSP_7;
set FirmInfo;
if Nyprc = Edyear-Styear+1;
if Nyprc > 90;
group = 0;
if Nydiv > 0 then group = 1;
run;

proc sort;
by group;
run;

proc means n sum;
var nyprc;
by group;
output out=TB1_CF7 n=nf sum=nfy;
run;
title "Basic Information for Firm age >90";

Data TB1_CF7;
set TB1_CF7;
Cat = 'Continuous_ge90';
run;

Data TB1;
set TB1 TB1_CF7;
if group = 1 then Div = 'Yes';
if group = 0 then Div = 'No';
label nf = '# of Firms';
label nfy = '# of Firm Years';
drop group _type_ _freq_;
run;

PROC EXPORT DATA= WORK.Tb1 
            OUTFILE= "&Dir\Output_Table1.XLXS" 
            DBMS=xlsx REPLACE;
     SHEET="Summary"; 
RUN;

proc freq data=FirmInfo;
table nyprc;
run;

/********************************************************
Basic information for each Firm age
********************************************************/
data Cnt_CRSP;
set FirmInfo;
if Nyprc = Edyear-Styear+1;
group = 0;
if Nydiv > 0 then group = 1;
run;

proc sort;
by group Nyprc;
run;
title "Basic information for each Firm age";
proc means noprint;
by group Nyprc;
output out=BI_Firmage n= sum=/autoname;
run;
