/*****************************************************
Panel Analysis for Table 2

last update: 2022-08-27
Yongok Choi(choiyongok@gmail.com)
Note: We assume r=0.054.
*****************************************************/

%let Dir D:\Dropbox\Research\Samuelson\Submit\Table2;

PROC IMPORT OUT= WORK.Obs_Info 
            DATAFILE= "&Dir\ObsInfo.txt" 
            DBMS=TAB REPLACE;
     GETNAMES=YES;
     DATAROW=2; 
RUN;


/******************************************************
Balanced Panel Analysis: one-way fixed effect model
******************************************************/
%macro B_panel(p=10);

PROC IMPORT OUT= WORK.Reg_gen_&p
            DATAFILE= "&Dir\Reg_gen_&p..txt" 
            DBMS=TAB REPLACE;
     GETNAMES=YES;
     DATAROW=2; 
RUN;

Data Balanced&p;
merge Sd.Reg_gen_&p obs_info;
by permno;
if gd = . then delete;
if obs = 93;
run;

proc sort data=Balanced&p;
by Permno Year;
run;

title 'Balanced Panel Analysis: one-way fixed effect model';
proc panel data=Balanced&p printfixed;
id Permno Year;
model gd = dp/ fixone neweywest;
run;
title;
%mend;

%B_panel(p=10);
%B_panel(p=20);
%B_panel(p=30);
%B_panel(p=50);

/******************************************************
Unbalanced Panel Analysis: one-way fixed effect model
******************************************************/
%macro UB_panel(p=10);
Data UNBalanced&p;
merge Reg_gen_&p obs_info;
by permno;
if gd = . then delete;
if obs >= &p+10;
if year >=1926;
run;

proc freq;
table obs;
run;

title "Unbalanced Panel Analysis: one-way fixed effect model";
proc panel data=Unbalanced&p;
id Permno Year;
model gd = dp/ fixone neweywest;
run;
title;
%mend;

%UB_panel(p=10);
%UB_panel(p=20);
%UB_panel(p=30);
%UB_panel(p=50);

/******************************************************
Unbalanced Panel Analysis (posive dividend): one-way fixed effect model
******************************************************/
%macro UB_panel_dpos(p=10);

PROC IMPORT OUT= WORK.Reg_gen_&p._dpos
            DATAFILE= "&Dir\Reg_gen_&p._dpos.txt" 
            DBMS=TAB REPLACE;
     GETNAMES=YES;
     DATAROW=2; 
RUN;

Data UNBalanced&p;
merge Reg_gen_&p._dpos obs_info;
by permno;
if gd = . then delete;
if obs >= &p+10;
if year >=1926;
run;

proc freq;
table obs;
run;

title "Unbalanced Panel Analysis: one-way fixed effect model";
proc panel data=Unbalanced&p;
id Permno Year;
model gd = dp/ fixone neweywest;
run;
title;
%mend;

%UB_panel_dpos(p=10);
%UB_panel_dpos(p=20);
%UB_panel_dpos(p=30);
%UB_panel_dpos(p=50);

