clc;clear;

for K = [10 20 30 50]
filename = strcat('ind_firm_regdataK',num2str(K),'.txt');

ind_firm_data = readtable(filename);

firmID = unique(ind_firm_data.PERMNO);
nf  = size(firmID,1);

Rst = zeros(nf,8);

for i=1:nf
    TP_ID = ind_firm_data.PERMNO == firmID(i,1);

    TP_y  = ind_firm_data.GD(TP_ID);
    TP_X  = ind_firm_data.DP(TP_ID);
    try
    [Cov,se,coeff] = hac(TP_X,TP_y);

    Rst(i,1) = firmID(i,1);
    Rst(i,2) = size(TP_y,1);
    Rst(i,3) = coeff(1,1);
    Rst(i,4) = se(1,1);
    Rst(i,5) = coeff(2,1);
    Rst(i,6) = se(2,1);
    Rst(i,7) = 1;
    catch
        Rst(i,7) = -1;
        Rst(i,1) = firmID(i,1);
        Rst(i,2) = size(TP_y,1);        
        Rst(i,3:6) = NaN;  
    end
end

FN_Rst = table();
FN_Rst.PERMNO   = Rst(:,1);
FN_Rst.Nobs     = Rst(:,2);
FN_Rst.alpha_hat= Rst(:,3);
FN_Rst.alpha_se = Rst(:,4);
FN_Rst.beta_hat = Rst(:,5);
FN_Rst.beta_se  = Rst(:,6);
FN_Rst.pvalue   = (1-tcdf(abs((Rst(:,5)+1)./Rst(:,6)),Rst(:,2)))*2;
FN_Rst.exitflag = Rst(:,7);

FN_Rst(FN_Rst.exitflag<=0,:) = [];

filename = 'Output_Table3.xlsx';
sheetname = strcat('K',num2str(K));
writetable(FN_Rst,filename,'Sheet',sheetname,'WriteVariableNames',true);
end

Ks          = zeros(4,1);
Nobs        = zeros(4,1);
beta_mean   = zeros(4,1);
beta_std    = zeros(4,1);
H_size1     = zeros(4,1);
H_size5     = zeros(4,1);
H_size10    = zeros(4,1);
alpha_mean  = zeros(4,1);
alpha_std   = zeros(4,1);
i = 1;
for K = [10 20 30 50]
filename = 'Output_Table3.xlsx';
sheetname = strcat('K',num2str(K));
Est_Rst = readtable(filename,Sheet=sheetname);

% Winsorizing
Valid_alpha = (Est_Rst.alpha_hat>prctile(Est_Rst.alpha_hat,2.5))&(Est_Rst.alpha_hat<prctile(Est_Rst.alpha_hat,97.5));
Valid_beta = (Est_Rst.beta_hat>prctile(Est_Rst.beta_hat,2.5))&(Est_Rst.beta_hat<prctile(Est_Rst.beta_hat,97.5));
Valid   = Valid_alpha&Valid_beta;

Ks(i,1)             = K;
Nobs(i,1)        = sum(Valid);
beta_mean(i,1)   = mean(Est_Rst.beta_hat(Valid));
beta_std(i,1)    = std(Est_Rst.beta_hat(Valid));
H_size1(i,1)     = mean(Est_Rst.pvalue(Valid)<0.01);
H_size5(i,1)     = mean(Est_Rst.pvalue(Valid)<0.05);
H_size10(i,1)    = mean(Est_Rst.pvalue(Valid)<0.10);
alpha_mean(i,1)  = mean(Est_Rst.alpha_hat(Valid));
alpha_std(i,1)   = std(Est_Rst.alpha_hat(Valid));
i = i + 1;
end

Table3 = table();
Table3.K = Ks;
Table3.Nobs = Nobs;
Table3.beta_mean = beta_mean;
Table3.beta_std  = beta_std;
Table3.H_size1   = H_size1;
Table3.H_size5   = H_size5;
Table3.H_size10  = H_size10;
Table3.alpha_mean = alpha_mean;
Table3.alpha_std = alpha_std;

writetable(Table3,'Output_Table3.xlsx','Sheet','Table3','WriteVariableNames',true);



