function [Val,opt_q] = HQ_Pr(D,s,max_q,k)

%Name:      HQ_Pr(Hannan-Quinn procedure)
%Purpose:   Selecting optimal ar order
%Format:    [Val,opt_q] = HQ(D,s,max_q,k)
%input:     D: time series
%           s: integration order
%           max_q: maximum AR order
%           k: constant for HQ statistic(k=1.001, Hannan and Quinn(1979, p. 194))
%output:    Val: mimimum HQ statistic Value
%           opt_q: optimal order for AR
%
%2011-01-22             By Yongok Choi
%Version 1.0 

if s > 0;
    Y = diff(D,s);
else
    Y = D;
end

HQ_stat = zeros(max_q,1);

for tmp_q = 1:1:max_q;
    Tmp_Y = Y(tmp_q+1:end);
    Tmp_X = ones(length(Tmp_Y),1);
    for order=1:1:tmp_q;
        Tmp_X = [Tmp_X Y(tmp_q+1-order:end-order,:)];
    end
    b       = (Tmp_X'*Tmp_X)\Tmp_X'*Tmp_Y;
    v_h     = Tmp_Y - Tmp_X*b;
    s2_v_h  = sum(v_h.^2)/(length(Tmp_Y)-tmp_q-1);    
    HQ_stat(tmp_q,1) = log(s2_v_h) + 2*tmp_q*k/(length(Tmp_Y)+s)*log(log(length(Tmp_Y)+s));
    
end

[Val,opt_q] = min(HQ_stat);

return


