% Efficient Market Test by Kenneth West(1988)
% When s = 0;
clc;clear;
tic;

CRSP_P= readtable('CRSP_P.txt');
CRSP_D= readtable('CRSP_D.txt');

D_Mat   = table2array(CRSP_D(:,[9	11	14	18	26	30	33	34	48	50	61	63	64	70	72	73	74	87	90	94	106	116	120	127	130	137	139	150	154	175	187	189	199	204	207	213	216	265	282]+1));
P_Mat   = table2array(CRSP_P(:,[9	11	14	18	26	30	33	34	48	50	61	63	64	70	72	73	74	87	90	94	106	116	120	127	130	137	139	150	154	175	187	189	199	204	207	213	216	265	282]+1));

[nrow,ncol] = size(D_Mat);

Est_rst_T1A_coef    = zeros(ncol,2);
Est_rst_T1A_se      = zeros(ncol,2);
Est_rst_T1B_coef    = zeros(ncol,6);
Est_rst_T1B_se      = zeros(ncol,6);
Est_rst_T2_coef     = zeros(ncol,6);
Est_rst_T2_se       = zeros(ncol,5);

for stock = 1:1:ncol

D       = D_Mat(D_Mat(:,stock)>=0,stock); 
P       = P_Mat(P_Mat(:,stock)>=0,stock);

s   = 0;

% Optimal AR order 
[Val,opt_q] = HQ_Pr(D,s,4,1.001);
% opt_q = 4;
% Twostep estimation for the Eq (14)
% Getting Optimal Instruments

Y       = P(opt_q:end-1,1);
X       = P(opt_q+1:end,1)+D(opt_q+1:end,1);

inst    = ones(length(Y),1);
for i=1:1:opt_q
    inst = [inst D(opt_q+1-i:end-i,:)];
end
W       = inst/(inst'*inst)*inst'*X;

% IV estimation for the Eq (14)
b_IV    = (W'*X)\W'*Y;
u_hat   = Y - X*b_IV;
s2_hat  = u_hat'*u_hat/length(u_hat);
rho_hat = (u_hat(1:end-1,1)'*u_hat(2:end,1))/(u_hat(1:end-1,1)'*u_hat(1:end-1,1));
e_hat   = u_hat(2:end,1)-rho_hat*u_hat(1:end-1,1);
s2_e_hat= e_hat'*e_hat/length(e_hat);
se_rho  = sqrt(s2_e_hat/(u_hat(1:end-1,1)'*u_hat(1:end-1,1)));

se_b_IV = sqrt(inv(W'*X)*(W'*W)*inv(W'*X)*s2_hat);

%% OLS estimation for the Eq (15)

Tmp_Y   = D(opt_q+1:end);
Tmp_X   = ones(length(Tmp_Y),1);
for order=1:1:opt_q;
    Tmp_X = [Tmp_X D(opt_q+1-order:end-order,:)];
end
pi      = (Tmp_X'*Tmp_X)\Tmp_X'*Tmp_Y;
v_hat   = Tmp_Y - Tmp_X*pi;
s2_v_h  = sum(v_hat.^2)/(length(Y)-opt_q-1);
rho_v_hat = (v_hat(1:end-1,1)'*v_hat(2:end,1))/(v_hat(1:end-1,1)'*v_hat(1:end-1,1));
e_v_hat   = v_hat(2:end,1)-rho_v_hat*v_hat(1:end-1,1);
s2_e_v_hat= e_v_hat'*e_v_hat/length(e_v_hat);
se_rho_v  = sqrt(s2_e_v_hat/(v_hat(1:end-1,1)'*v_hat(1:end-1,1)));

se_pi = sqrt(diag(inv(Tmp_X'*Tmp_X)*s2_v_h));

%% Calculating the statistic in (16)

Est_rst = zeros(2,5);

h_tt = zeros(opt_q+4,opt_q+4);

Tmp_sum = 0;
for i=1:1:opt_q;
    Tmp_sum = b_IV^i*pi(1+i,1)+Tmp_sum;
end

delta_1 = 1/(1-Tmp_sum);

Stat_16 = s2_v_h*delta_1^2-s2_hat/b_IV^2;
PVinP   = -100*Stat_16/(s2_hat/b_IV^2);      %percentage of variance in the price process

Est_rst_T2_coef(stock,1) = b_IV;
Est_rst_T2_coef(stock,2) = delta_1;
Est_rst_T2_coef(stock,3) = s2_hat;
Est_rst_T2_coef(stock,4) = s2_v_h;
Est_rst_T2_coef(stock,5) = Stat_16;
Est_rst_T2_coef(stock,6) = PVinP;
Z   = Tmp_X;

Zu  = [];

for i=1:1:1+opt_q;
    Zu = [Zu Z(:,i).*u_hat];
end;

S_z_hat = Zu'*Zu/length(Z);

N   = X;

NZ  = [];

for i=1:1:1+opt_q;
    NZ = [NZ Z(:,i).*N];
end


T   = length(NZ);

h_tt(1,1) = -sum(NZ)*inv(S_z_hat)*sum(NZ)';

h_tt_ZdZ = Z'*Z;

h_tt(2:opt_q+2,2:opt_q+2) = h_tt_ZdZ;

h_tt(opt_q+3,1) = 2 * sum((Y- N*b_IV).*N);
h_tt(opt_q+3,opt_q+3) = T;

dmZ = v_hat;

for i = 1:1:opt_q+1;
    h_tt(opt_q+4,1+i) = 2*sum(dmZ.*Z(:,i));
end

h_tt(opt_q+4,opt_q+4) = T;

h_tt =h_tt/T;

h_t = zeros(T,opt_q+4);

h_t(:,1) = sum(NZ)*inv(S_z_hat)*Zu';

for i=1:1:opt_q+1;
    h_t(:,i+1) = dmZ.*Z(:,i);
end

h_t(:,opt_q+3) = s2_hat-(Y- N*b_IV).^2;
h_t(:,opt_q+4) = s2_v_h - dmZ.^2;

% Estimation of S

hat_S = h_t'*h_t/T;
m = 11;
for i=1:1:m;
    hat_S = hat_S + (m+1-i)/(m+1)*( h_t(i+1:end,:)'*h_t(1:end-i,:)/T +   h_t(1:end-i,:)'*h_t(i+1:end,:)/T);
end

V_hat = h_tt\hat_S/h_tt';

%% Asymptotic Variance

numerator = 0;
denominator = 1;

for i=1:1:opt_q;
    numerator = numerator - pi(i+1,1)*i*b_IV^(i-1);
    denominator = denominator -b_IV^i *pi(i+1,1);
end

%b
df_1    = zeros(opt_q+4,1);
df_1(1,1) = 1;

Est_rst_T2_se(stock,1) = sqrt(df_1'*V_hat*df_1/T);

%delta_1

df_2    = zeros(opt_q+4,1);
df_2(1,1) = -numerator/denominator;
for i=1:1:opt_q;
    df_2(i+2,1) = b_IV^i/denominator^2;
end

Est_rst_T2_se(stock,2) = sqrt(df_2'*V_hat*df_2/T);

%s2_u

df_3    = zeros(opt_q+4,1);
df_3(3+opt_q,1) = 1;

Est_rst_T2_se(stock,3) = sqrt(df_3'*V_hat*df_3/T);

%s2_v

df_4    = zeros(opt_q+4,1);
df_4(4+opt_q,1) = 1;

Est_rst_T2_se(stock,4) = sqrt(df_4'*V_hat*df_4/T);

%stat(16)
df      = zeros(opt_q+4,1);

df(1,1) = 2*s2_hat/b_IV^3 - s2_v_h*numerator/denominator^2;

for i=1:1:opt_q;
    df(i+2,1)=b_IV^i *s2_v_h/denominator^2 ;
end

df(opt_q+3,1) = -1/b_IV^2;

df(opt_q+4,1) = 1/denominator;

Est_rst_T2_se(stock,5) = sqrt(df'*V_hat*df/T);

Est_rst_T1A_coef(stock,1)     = Est_rst_T2_coef(stock,1);
Est_rst_T1A_se(stock,1)       = Est_rst_T2_se(stock,1);

Est_rst_T1A_coef(stock,2)     = rho_hat;
Est_rst_T1A_se(stock,2)       = se_rho;


%mu
df_1    = zeros(opt_q+4,1);
df_1(2,1) = 1;
Est_rst_T1B_coef(stock,1) = pi(1,1);
Est_rst_T1B_se(stock,1) = sqrt(df_1'*V_hat*df_1/T);
%pi_1,...pi_q
for i=1:1:opt_q
    df_tmp  = zeros(opt_q+4,1);
    df_tmp(2+i,1) = 1;
    Est_rst_T1B_coef(stock,1+i) = pi(1+i,1);
    Est_rst_T1B_se(stock,1+i) = sqrt(df_tmp'*V_hat*df_tmp/T);    
end
Est_rst_T1B_coef(stock,6)     = rho_v_hat;
Est_rst_T1B_se(stock,6)     = se_rho_v;

end

Final_rst_s0 = [Est_rst_T2_coef(:,5) Est_rst_T2_coef(:,6) Est_rst_T2_se(:,5) Est_rst_T2_coef(:,5)./Est_rst_T2_se(:,5)];


%% s= 1

D_Mat   = table2array(CRSP_D(:,[1	2	3	4	5	6	7	8	10	12	13	15	16	17	19	20	21	22	23	24	25	27	28	29	31	32	35	36	37	38	39	40	41	42	43	44	45	46	47	49	51	52	53	54	55	56	57	58	59	60	62	65	66	67	68	69	71	75	76	77	78	79	80	81	82	83	84	85	86	88	89	91	92	93	95	96	97	98	99	100	101	102	103	104	105	107	108	109	110	111	112	113	114	115	117	118	119	121	122	123	124	125	126	128	129	131	132	133	134	135	136	138	140	141	142	143	144	145	146	147	148	149	151	152	153	155	156	157	158	159	160	161	162	163	164	165	166	167	168	169	170	171	172	173	174	176	177	178	179	180	181	182	183	184	185	186	188	190	191	192	193	194	195	196	197	198	200	201	202	203	205	206	208	209	210	211	212	214	215	217	218	219	220	221	222	223	224	225	226	227	228	229	230	231	232	233	234	235	236	237	238	239	240	241	242	243	244	245	246	247	248	249	250	251	252	253	254	255	256	257	258	259	260	261	262	263	264	266	267	268	269	270	271	272	273	274	275	276	277	278	279	280	281	283	284	285	286	287]+1));
P_Mat   = table2array(CRSP_P(:,[1	2	3	4	5	6	7	8	10	12	13	15	16	17	19	20	21	22	23	24	25	27	28	29	31	32	35	36	37	38	39	40	41	42	43	44	45	46	47	49	51	52	53	54	55	56	57	58	59	60	62	65	66	67	68	69	71	75	76	77	78	79	80	81	82	83	84	85	86	88	89	91	92	93	95	96	97	98	99	100	101	102	103	104	105	107	108	109	110	111	112	113	114	115	117	118	119	121	122	123	124	125	126	128	129	131	132	133	134	135	136	138	140	141	142	143	144	145	146	147	148	149	151	152	153	155	156	157	158	159	160	161	162	163	164	165	166	167	168	169	170	171	172	173	174	176	177	178	179	180	181	182	183	184	185	186	188	190	191	192	193	194	195	196	197	198	200	201	202	203	205	206	208	209	210	211	212	214	215	217	218	219	220	221	222	223	224	225	226	227	228	229	230	231	232	233	234	235	236	237	238	239	240	241	242	243	244	245	246	247	248	249	250	251	252	253	254	255	256	257	258	259	260	261	262	263	264	266	267	268	269	270	271	272	273	274	275	276	277	278	279	280	281	283	284	285	286	287]+1));

[nrow ncol] = size(D_Mat);

Est_rst_T1A_coef    = zeros(ncol,2);
Est_rst_T1A_se      = zeros(ncol,2);
Est_rst_T1B_coef    = zeros(ncol,6);
Est_rst_T1B_se      = zeros(ncol,6);
Est_rst_T2_coef     = zeros(ncol,6);
Est_rst_T2_se       = zeros(ncol,5);

for stock = 1:1:ncol

D       = D_Mat(D_Mat(:,stock)>=0,stock); 
P       = P_Mat(P_Mat(:,stock)>=0,stock);

s   = 1;

% Optimal AR order 
%[Val,opt_q] = HQ_Pr(D,s,4,1.001);
opt_q =4;

P       = P(s+1:end);

% Twostep estimation for the Eq (14)
% Getting Optimal Instruments

Y       = P(opt_q:end-1,1);
X       = P(opt_q+1:end,1)+D(opt_q+1+s:end,1);

D       = diff(D,s);

inst    = ones(length(Y),1);
for i=1:1:opt_q
    inst = [inst D(opt_q+1-i:end-i,:)];
end
W       = inst/(inst'*inst)*inst'*X;

% IV estimation for the Eq (14)
b_IV    = (W'*X)\W'*Y;
u_hat   = Y - X*b_IV;
s2_hat  = u_hat'*u_hat/length(u_hat);
rho_hat = (u_hat(1:end-1,1)'*u_hat(2:end,1))/(u_hat(1:end-1,1)'*u_hat(1:end-1,1));
e_hat   = u_hat(2:end,1)-rho_hat*u_hat(1:end-1,1);
s2_e_hat= e_hat'*e_hat/length(e_hat);
se_rho  = sqrt(s2_e_hat/(u_hat(1:end-1,1)'*u_hat(1:end-1,1)));

se_b_IV = sqrt((W'*W)\s2_hat);

%% OLS estimation for the Eq (15)

Tmp_Y   = D(opt_q+1:end);
Tmp_X   = ones(length(Tmp_Y),1);
for order=1:1:opt_q;
    Tmp_X = [Tmp_X D(opt_q+1-order:end-order,:)];
end
pi      = (Tmp_X'*Tmp_X)\Tmp_X'*Tmp_Y;
v_hat   = Tmp_Y - Tmp_X*pi;
s2_v_h  = sum(v_hat.^2)/(length(Y)-opt_q-1);
rho_v_hat = (v_hat(1:end-1,1)'*v_hat(2:end,1))/(v_hat(1:end-1,1)'*v_hat(1:end-1,1));
e_v_hat   = v_hat(2:end,1)-rho_v_hat*v_hat(1:end-1,1);
s2_e_v_hat= e_v_hat'*e_v_hat/length(e_v_hat);
se_rho_v  = sqrt(s2_e_v_hat/(v_hat(1:end-1,1)'*v_hat(1:end-1,1)));

se_pi = sqrt(diag(inv(Tmp_X'*Tmp_X)*s2_v_h));

%% Calculating the statistic in (16)

Est_rst = zeros(2,5);

h_tt = zeros(opt_q+4,opt_q+4);

Tmp_sum = 0;
for i=1:1:opt_q;
    Tmp_sum = b_IV^i*pi(1+i,1)+Tmp_sum;
end

delta_1 = 1/((1-b_IV)*(1-Tmp_sum));

Stat_16 = s2_v_h*delta_1^2-s2_hat/b_IV^2;
PVinP   = -100*Stat_16/(s2_hat/b_IV^2);      %percentage of variance in the price process
Est_rst_T2_coef(stock,1) = b_IV;
Est_rst_T2_coef(stock,2) = delta_1;
Est_rst_T2_coef(stock,3) = s2_hat;
Est_rst_T2_coef(stock,4) = s2_v_h;
Est_rst_T2_coef(stock,5) = Stat_16;
Est_rst_T2_coef(stock,6) = PVinP;

Z   = Tmp_X;

Zu  = [];

for i=1:1:1+opt_q;
    Zu = [Zu Z(:,i).*u_hat];
end;

S_z_hat = Zu'*Zu/length(Z);

N   = X;

NZ  = [];

for i=1:1:1+opt_q;
    NZ = [NZ Z(:,i).*N];
end


T   = length(NZ);

h_tt(1,1) = -sum(NZ)*inv(S_z_hat)*sum(NZ)'/T^(2*s);



h_tt_ZdZ = Z'*Z;

h_tt(2:opt_q+2,2:opt_q+2) = h_tt_ZdZ;

h_tt(opt_q+3,1) = 2 * sum((Y- N*b_IV).*N);
h_tt(opt_q+3,opt_q+3) = T;

dmZ = v_hat;

for i = 1:1:opt_q+1;
    h_tt(opt_q+4,1+i) = 2*sum(dmZ.*Z(:,i));
end

h_tt(opt_q+4,opt_q+4) = T;

h_tt =h_tt/T;
h_tt(:,1) = h_tt(:,1)/T;

h_t = zeros(T,opt_q+4);

h_t(:,1) = sum(NZ)*inv(S_z_hat)*Zu'/T^(2*s);

for i=1:1:opt_q+1;
    h_t(:,i+1) = dmZ.*Z(:,i);
end

h_t(:,opt_q+3) = s2_hat-(Y- N*b_IV).^2;
h_t(:,opt_q+4) = s2_v_h - dmZ.^2;

% Estimation of S

hat_S = h_t'*h_t/T;
m = 11;
for i=1:1:m;
    hat_S = hat_S + (m+1-i)/(m+1)*( h_t(i+1:end,:)'*h_t(1:end-i,:)/T +   h_t(1:end-i,:)'*h_t(i+1:end,:)/T);
end

V_hat = h_tt\hat_S/h_tt';

Norm = zeros(opt_q+4,opt_q+4);

Norm(1,1) = T*sqrt(T);

for i=2:1:opt_q+4;
    Norm(i,i) = sqrt(T);
end

Norm_V_hat = inv(Norm)*V_hat*inv(Norm);

%% Asymptotic Variance

numerator = 0;
denominator = 1;

for i=1:1:opt_q;
    numerator = numerator - pi(i+1,1)*i*b_IV^(i-1);
    denominator = denominator -b_IV^i *pi(i+1,1);
end

%b
df_1    = zeros(opt_q+4,1);
df_1(1,1) = 1;

Est_rst_T2_se(stock,1) = sqrt(df_1'*Norm_V_hat*df_1);

%delta_1

df_2    = zeros(opt_q+4,1);
df_2(1,1) = 1/(denominator*(1-b_IV)^2)-numerator/((1-b_IV)*denominator^2);
for i=1:1:opt_q;
    df_2(i+2,1) = b_IV^i/((1-b_IV)*denominator^2);
end

Est_rst_T2_se(stock,2) = sqrt(df_2'*Norm_V_hat*df_2);

%s2_u

df_3    = zeros(opt_q+4,1);
df_3(3+opt_q,1) = 1;

Est_rst_T2_se(stock,3) = sqrt(df_3'*Norm_V_hat*df_3);

%s2_v

df_4    = zeros(opt_q+4,1);
df_4(4+opt_q,1) = 1;

Est_rst_T2_se(stock,4) = sqrt(df_4'*Norm_V_hat*df_4);


df      = zeros(opt_q+4,1);

df(1,1) = 2*s2_hat/b_IV^3 - s2_v_h*numerator/(denominator^2*(1-b_IV))+ s2_v_h/(denominator*(1-b_IV)^2);

for i=1:1:opt_q;
    df(i+2,1)=b_IV^i *s2_v_h/(denominator^2*(1-b_IV)) ;
end

df(opt_q+3,1) = -1/b_IV^2;

df(opt_q+4,1) = 1/(denominator*(1-b_IV));

Est_rst_T2_se(stock,5) = sqrt(df'*Norm_V_hat*df);

Est_rst_T1A_coef(stock,1)     = Est_rst_T2_coef(stock,1);
Est_rst_T1A_se(stock,1)       = Est_rst_T2_se(stock,1);

Est_rst_T1A_coef(stock,2)     = rho_hat;
Est_rst_T1A_se(stock,2)       = se_rho;


%mu
df_1    = zeros(opt_q+4,1);
df_1(2,1) = 1;
Est_rst_T1B_coef(stock,1) = pi(1,1);
Est_rst_T1B_se(stock,1) = sqrt(df_1'*V_hat*df_1/T);
%pi_1,...pi_q
for i=1:1:opt_q;
    df_tmp  = zeros(opt_q+4,1);
    df_tmp(2+i,1) = 1;
    Est_rst_T1B_coef(stock,1+i) = pi(1+i,1);
    Est_rst_T1B_se(stock,1+i) = sqrt(df_tmp'*Norm_V_hat*df_tmp);    
end
Est_rst_T1B_coef(stock,6)     = rho_v_hat;
Est_rst_T1B_se(stock,6)     = se_rho_v;

end

Final_rst_s1 = [Est_rst_T2_coef(:,5) Est_rst_T2_coef(:,6) Est_rst_T2_se(:,5) Est_rst_T2_coef(:,5)./Est_rst_T2_se(:,5)];

Final_rst = [Final_rst_s0;Final_rst_s1];

Summary = zeros(1,6);
Summary(1,1) = size(Final_rst,1);
Summary(1,2) = mean(Final_rst(:,1));
Summary(1,3) = mean(Final_rst(:,2));
if Summary(1,3) < 0
    Summary(1,3) = NaN;
end
Summary(1,4) = sum(Final_rst(:,4) >= norminv(0.01,0,1));
Summary(1,5) = sum(Final_rst(:,4) >= norminv(0.05,0,1));
Summary(1,6) = sum(Final_rst(:,4) >= norminv(0.1,0,1));