% S&P500 since 1926 
% When s = 1;
% Note: 
% 1. Variables' name and equation numbers are from West(1988).
% 2. "Final_rst" is the results for the main test statistic. 

clc;clear;

SP500_Data = readtable('SP500_RealPD.txt');

D   = SP500_Data.Dividend(56:end,1);
preD= SP500_Data.Dividend(56:end,1);
P   = SP500_Data.Price(56:end,1);

s   = 1;

% Optimal AR order 
 [Val,opt_q] = HQ_Pr(D,s,4,1.001);
%opt_q = 4;

D       = diff(D,s);
P       = P(s+1:end);

% Twostep estimation for the Eq (14)
% Getting Optimal Instruments

Y       = P(opt_q:end-1,1);
X       = P(opt_q+1:end,1)+preD(opt_q+1+s:end,1);

inst    = ones(length(Y),1);
for i=1:1:opt_q
    inst = [inst D(opt_q+1-i:end-i,:)];
end
W       = inst/(inst'*inst)*inst'*X;

% IV estimation for the Eq (14)
b_IV    = (W'*X)\W'*Y;
u_hat   = Y - X*b_IV;
s2_hat  = u_hat'*u_hat/length(u_hat);
rho_hat = (u_hat(1:end-1,1)'*u_hat(2:end,1))/(u_hat(1:end-1,1)'*u_hat(1:end-1,1));
e_hat   = u_hat(2:end,1)-rho_hat*u_hat(1:end-1,1);
s2_e_hat= e_hat'*e_hat/length(e_hat);
se_rho  = sqrt(s2_e_hat/(u_hat(1:end-1,1)'*u_hat(1:end-1,1)));

se_b_IV = sqrt(inv(W'*X)*(W'*W)*inv(W'*X)*s2_hat);

% OLS estimation for the Eq (15)

Tmp_Y   = D(opt_q+1:end);
Tmp_X   = ones(length(Tmp_Y),1);
for order=1:1:opt_q;
    Tmp_X = [Tmp_X D(opt_q+1-order:end-order,:)];
end
pi      = (Tmp_X'*Tmp_X)\Tmp_X'*Tmp_Y;
v_hat   = Tmp_Y - Tmp_X*pi;
s2_v_h  = sum(v_hat.^2)/(length(Y)-opt_q-1);
rho_v_hat = (v_hat(1:end-1,1)'*v_hat(2:end,1))/(v_hat(1:end-1,1)'*v_hat(1:end-1,1));
e_v_hat   = v_hat(2:end,1)-rho_v_hat*v_hat(1:end-1,1);
s2_e_v_hat= e_v_hat'*e_v_hat/length(e_v_hat);
se_rho_v  = sqrt(s2_e_v_hat/(v_hat(1:end-1,1)'*v_hat(1:end-1,1)));

se_pi = sqrt(diag(inv(Tmp_X'*Tmp_X)*s2_v_h));

% Calculating the statistic in (16)

Est_rst = zeros(2,5);

h_tt = zeros(opt_q+4,opt_q+4);

Tmp_sum = 0;
for i=1:1:opt_q
    Tmp_sum = b_IV^i*pi(1+i,1)+Tmp_sum;
end

delta_1 = 1/((1-b_IV)*(1-Tmp_sum));

Stat_16 = s2_v_h*delta_1^2-s2_hat/b_IV^2;
Stat_17 = -100*(Stat_16)/(s2_hat/b_IV^2);

Est_rst(1,1) = b_IV;
Est_rst(1,2) = delta_1;
Est_rst(1,3) = s2_hat;
Est_rst(1,4) = s2_v_h;
Est_rst(1,5) = Stat_16;

Z   = Tmp_X;

Zu  = [];

for i=1:1:1+opt_q
    Zu = [Zu Z(:,i).*u_hat];
end

S_z_hat = Zu'*Zu/length(Z);

N   = X;

NZ  = [];

for i=1:1:1+opt_q
    NZ = [NZ Z(:,i).*N];
end


T   = length(NZ);

h_tt(1,1) = -sum(NZ)*inv(S_z_hat)*sum(NZ)'/T^(2*s);

h_tt_ZdZ  = zeros(1+opt_q,1+opt_q);

for i=1:1:1+opt_q
    for j=1:1:1+opt_q
        h_tt_ZdZ(i,j) = -sum(Z(:,i).*(D(1+opt_q:end,1)-Z(:,j)));
    end
end

h_tt(2:opt_q+2,2:opt_q+2) = h_tt_ZdZ;

h_tt(opt_q+3,1) = 2 * sum((Y- N*b_IV).*N);
h_tt(opt_q+3,opt_q+3) = T;

dmZ = D(1+opt_q:end,1)-Z*pi;

for i = 1:1:opt_q+1
    h_tt(opt_q+4,1+i) = 2*sum(dmZ.*Z(:,i));
end

h_tt(opt_q+4,opt_q+4) = T;

h_tt =h_tt/T;
h_tt(:,1) = h_tt(:,1)/T;

h_t = zeros(T,opt_q+4);

h_t(:,1) = sum(NZ)*inv(S_z_hat)*Zu'/T^(2*s);

for i=1:1:opt_q+1
    h_t(:,i+1) = dmZ.*Z(:,i);
end

h_t(:,opt_q+3) = s2_hat-(Y- N*b_IV).^2;
h_t(:,opt_q+4) = s2_v_h - dmZ.^2;

% Estimation of S

hat_S = h_t'*h_t/T;
m = 11;
for i=1:1:m
    hat_S = hat_S + (m+1-i)/(m+1)*( h_t(i+1:end,:)'*h_t(1:end-i,:)/T +   h_t(1:end-i,:)'*h_t(i+1:end,:)/T);
end

V_hat = h_tt\hat_S/h_tt';

Norm = zeros(opt_q+4,opt_q+4);

Norm(1,1) = T*sqrt(T);

for i=2:1:opt_q+4
    Norm(i,i) = sqrt(T);
end

Norm_V_hat = inv(Norm)*V_hat*inv(Norm);

% Asymptotic Variance

numerator = 0;
denominator = 1;

for i=1:1:opt_q
    numerator = numerator - pi(i+1,1)*i*b_IV^(i-1);
    denominator = denominator -b_IV^i *pi(i+1,1);
end

%b
df_1    = zeros(opt_q+4,1);
df_1(1,1) = 1;

Est_rst(2,1) = sqrt(df_1'*Norm_V_hat*df_1);

%delta_1

df_2    = zeros(opt_q+4,1);
df_2(1,1) = 1/(denominator*(1-b_IV)^2)-numerator/((1-b_IV)*denominator^2);
for i=1:1:opt_q
    df_2(i+2,1) = b_IV^i/((1-b_IV)*denominator^2);
end

Est_rst(2,2) = sqrt(df_2'*Norm_V_hat*df_2);

%s2_u

df_3    = zeros(opt_q+4,1);
df_3(3+opt_q,1) = 1;

Est_rst(2,3) = sqrt(df_3'*Norm_V_hat*df_3);

%s2_v

df_4    = zeros(opt_q+4,1);
df_4(4+opt_q,1) = 1;

Est_rst(2,4) = sqrt(df_4'*Norm_V_hat*df_4);


df      = zeros(opt_q+4,1);

df(1,1) = 2*s2_hat/b_IV^3 - s2_v_h*numerator/(denominator^2*(1-b_IV))+ s2_v_h/(denominator*(1-b_IV)^2);

for i=1:1:opt_q
    df(i+2,1)=b_IV^i *s2_v_h/(denominator^2*(1-b_IV)) ;
end

df(opt_q+3,1) = -1/b_IV^2;

df(opt_q+4,1) = 1/(denominator*(1-b_IV));

Est_rst(2,5) = sqrt(df'*Norm_V_hat*df);

Est_rst_tA = zeros(2,2);

Est_rst_tA(1,1)     = Est_rst(1,1);
Est_rst_tA(2,1)     = Est_rst(2,1);

Est_rst_tA(1,2)     = rho_hat;
Est_rst_tA(2,2)     = se_rho;

Est_rst_tB = zeros(2,6);
%mu
df_1    = zeros(opt_q+4,1);
df_1(2,1) = 1;
Est_rst_tB(1,1) = pi(1,1);
Est_rst_tB(2,1) = sqrt(df_1'*V_hat*df_1/T);
%pi_1,...pi_q
df_tmp  = zeros(opt_q+4,1);
for i=1:1:opt_q
    df_tmp(2+i,1) = 1;
    Est_rst_tB(1,1+i) = pi(1+i,1);
    Est_rst_tB(2,1+i) = sqrt(df_tmp'*Norm_V_hat*df_tmp);    
end
Est_rst_tB(1,6)     = rho_v_hat;
Est_rst_tB(2,6)     = se_rho_v;

Final_rst = [Est_rst(1,5) Stat_17];